@@ -8,6 +8,7 @@ from __future__ import annotations
 
 from loppers.extensions import EXTENSION_TO_LANGUAGE, get_language
 from loppers.source_utils import (
+    concatenate_files,
     extract_skeleton,
     find_files,
     get_skeleton,
@@ -15,10 +16,11 @@ from loppers.source_utils import (
 )
 
 __all__ = [
-    "get_skeleton",
-    "find_files",
-    "get_tree",
-    "extract_skeleton",
     "EXTENSION_TO_LANGUAGE",
+    "concatenate_files",
+    "extract_skeleton",
+    "find_files",
     "get_language",
+    "get_skeleton",
+    "get_tree",
 ]
