@@ -6,10 +6,11 @@ import tempfile
 import unittest
 from pathlib import Path
 
-from loppers import extract_skeleton, find_files, get_skeleton
-from loppers.loppers import SkeletonExtractor
 from binaryornot.check import is_binary
 
+from loppers import concatenate_files, extract_skeleton, find_files, get_skeleton
+from loppers.loppers import SkeletonExtractor
+
 
 class TestSkeletonExtractor(unittest.TestCase):
     """Test skeleton extraction."""
@@ -491,14 +492,13 @@ class TestFindFilesWithMixedTypes(unittest.TestCase):
             (tmppath / "script.py").write_text("def hello():\n    pass\n")
 
             # Find files
-            files = find_files([tmppath], recursive=False)
-            file_names = {f.name for f in files}
+            files = find_files(tmppath, recursive=False)
 
             # All text files should be included
-            self.assertIn("readme.md", file_names)
-            self.assertIn("config.json", file_names)
-            self.assertIn("data.yaml", file_names)
-            self.assertIn("script.py", file_names)
+            self.assertIn("readme.md", files)
+            self.assertIn("config.json", files)
+            self.assertIn("data.yaml", files)
+            self.assertIn("script.py", files)
 
     def test_find_files_excludes_binary_files(self) -> None:
         """Test that find_files excludes binary files."""
@@ -509,11 +509,10 @@ class TestFindFilesWithMixedTypes(unittest.TestCase):
             (tmppath / "text.txt").write_text("text content\n")
             (tmppath / "binary.bin").write_bytes(b"\x00\x01\x02\x03")
 
-            files = find_files([tmppath], recursive=False)
-            file_names = {f.name for f in files}
+            files = find_files(tmppath, recursive=False)
 
-            self.assertIn("text.txt", file_names)
-            self.assertNotIn("binary.bin", file_names)
+            self.assertIn("text.txt", files)
+            self.assertNotIn("binary.bin", files)
 
 
 class TestMixedFileTypes(unittest.TestCase):
@@ -585,16 +584,244 @@ class TestIgnorePatternsWithMixedFiles(unittest.TestCase):
             (tmppath / "main.js").write_text("console.log('hi');\n")
 
             files = find_files(
-                [tmppath],
+                tmppath,
                 recursive=True,
                 use_default_ignore=True,
             )
-            file_names = {f.name for f in files}
 
             # Files in node_modules should be excluded
-            self.assertIn("main.js", file_names)
-            self.assertNotIn("package.json", file_names)
-            self.assertNotIn("module.js", file_names)
+            self.assertIn("main.js", files)
+            self.assertNotIn("node_modules/package.json", files)
+            self.assertNotIn("node_modules/module.js", files)
+
+
+class TestConcatenateFiles(unittest.TestCase):
+    """Test file concatenation."""
+
+    def test_concatenate_files_with_extraction(self) -> None:
+        """Test concatenating files with skeleton extraction."""
+        with tempfile.TemporaryDirectory() as tmpdir:
+            root = Path(tmpdir)
+
+            # Create test files
+            py_file = root / "script.py"
+            py_file.write_text(
+                "def hello():\n"
+                '    """Say hello."""\n'
+                "    print('Hello')\n"
+            )
+
+            js_file = root / "app.js"
+            js_file.write_text(
+                "function greet() {\n"
+                "    console.log('Hi');\n"
+                "}\n"
+            )
+
+            # Concatenate with extraction
+            result = concatenate_files(root, ["script.py", "app.js"], extract=True)
+
+            # Check headers
+            self.assertIn("--- script.py", result)
+            self.assertIn("--- app.js", result)
+
+            # Check skeleton extraction (no body)
+            self.assertIn("def hello", result)
+            self.assertNotIn("print", result)
+            self.assertIn("function greet", result)
+            self.assertNotIn("console.log", result)
+
+    def test_concatenate_files_without_extraction(self) -> None:
+        """Test concatenating files without skeleton extraction."""
+        with tempfile.TemporaryDirectory() as tmpdir:
+            root = Path(tmpdir)
+
+            py_file = root / "script.py"
+            py_file.write_text(
+                "def hello():\n"
+                "    print('Hello')\n"
+            )
+
+            # Concatenate without extraction
+            result = concatenate_files(root, ["script.py"], extract=False)
+
+            # Check full content is preserved
+            self.assertIn("print", result)
+            self.assertIn("def hello", result)
+
+    def test_concatenate_files_not_found_strict(self) -> None:
+        """Test error when file not found with ignore_not_found=False."""
+        with tempfile.TemporaryDirectory() as tmpdir:
+            root = Path(tmpdir)
+            with self.assertRaises(FileNotFoundError):
+                concatenate_files(root, ["missing.py"], ignore_not_found=False)
+
+    def test_concatenate_files_not_found_ignore(self) -> None:
+        """Test ignoring missing files with ignore_not_found=True."""
+        with tempfile.TemporaryDirectory() as tmpdir:
+            root = Path(tmpdir)
+
+            # Create one valid file
+            py_file = root / "script.py"
+            py_file.write_text("def hello():\n    pass\n")
+
+            # Concatenate with missing file, ignoring not found
+            result = concatenate_files(
+                root, ["script.py", "missing.py"], ignore_not_found=True
+            )
+
+            # Should only include the found file
+            self.assertIn("--- script.py", result)
+            self.assertNotIn("missing.py", result)
+
+    def test_concatenate_empty_paths(self) -> None:
+        """Test error when no paths provided."""
+        with tempfile.TemporaryDirectory() as tmpdir:
+            root = Path(tmpdir)
+            with self.assertRaises(ValueError):
+                concatenate_files(root, [])
+
+    def test_concatenate_invalid_root(self) -> None:
+        """Test error when root does not exist."""
+        with self.assertRaises(FileNotFoundError):
+            concatenate_files("/nonexistent/path", ["file.py"])
+
+    def test_concatenate_root_not_directory(self) -> None:
+        """Test error when root is not a directory."""
+        with tempfile.TemporaryDirectory() as tmpdir:
+            root = Path(tmpdir)
+            file_path = root / "file.txt"
+            file_path.write_text("content")
+
+            with self.assertRaises(NotADirectoryError):
+                concatenate_files(file_path, ["file.py"])
+
+
+class TestFileSizeFormatting(unittest.TestCase):
+    """Test file size formatting utility."""
+
+    def test_format_bytes(self):
+        """Test formatting of byte sizes."""
+        from loppers.source_utils import format_file_size
+
+        self.assertEqual(format_file_size(0), "0B")
+        self.assertEqual(format_file_size(100), "100B")
+        self.assertEqual(format_file_size(1023), "1023B")
+
+    def test_format_kilobytes(self):
+        """Test formatting of kilobyte sizes."""
+        from loppers.source_utils import format_file_size
+
+        self.assertEqual(format_file_size(1024), "1.0KB")
+        self.assertEqual(format_file_size(2048), "2.0KB")
+        self.assertEqual(format_file_size(1536), "1.5KB")  # 1.5KB
+
+    def test_format_megabytes(self):
+        """Test formatting of megabyte sizes."""
+        from loppers.source_utils import format_file_size
+
+        kb = 1024
+        mb = kb * 1024
+        self.assertEqual(format_file_size(mb), "1.0MB")
+        self.assertEqual(format_file_size(mb * 2), "2.0MB")
+        self.assertEqual(format_file_size(mb * 5 + kb * 300), "5.3MB")
+
+    def test_format_gigabytes(self):
+        """Test formatting of gigabyte sizes."""
+        from loppers.source_utils import format_file_size
+
+        kb = 1024
+        mb = kb * 1024
+        gb = mb * 1024
+        self.assertEqual(format_file_size(gb), "1.0GB")
+        self.assertEqual(format_file_size(gb * 2), "2.0GB")
+        self.assertEqual(format_file_size(gb + mb * 256), "1.2GB")
+
+
+class TestTreeWithSizes(unittest.TestCase):
+    """Test tree display with file sizes."""
+
+    def test_tree_as_str_with_sizes(self):
+        """Test tree_as_str with file sizes."""
+        from loppers.source_utils import tree_as_str
+
+        paths = ["file1.py", "dir1/file2.py", "dir1/file3.txt"]
+        file_sizes = {
+            "file1.py": 1024,
+            "dir1/file2.py": 2048,
+            "dir1/file3.txt": 512,
+        }
+        tree = tree_as_str(paths, file_sizes=file_sizes)
+        self.assertIn("file1.py  (1.0KB)", tree)
+        self.assertIn("file2.py  (2.0KB)", tree)
+        self.assertIn("file3.txt  (512B)", tree)
+
+    def test_tree_as_str_without_sizes(self):
+        """Test tree_as_str without file sizes."""
+        from loppers.source_utils import tree_as_str
+
+        paths = ["file1.py", "dir1/file2.py"]
+        tree = tree_as_str(paths, file_sizes=None)
+        self.assertIn("file1.py", tree)
+        self.assertIn("file2.py", tree)
+        # Should not have size info
+        self.assertNotIn("(", tree) or "." not in tree
+
+    def test_tree_as_str_with_collapse_and_sizes(self):
+        """Test tree_as_str with both collapse and sizes."""
+        from loppers.source_utils import tree_as_str
+
+        paths = ["main/java/com/example/Source.java"]
+        file_sizes = {"main/java/com/example/Source.java": 5120}
+        tree = tree_as_str(paths, collapse_single_dirs=True, file_sizes=file_sizes)
+        # Should collapse the nested path
+        self.assertIn("main/java/com/example", tree)
+        # Should show size
+        self.assertIn("(5.0KB)", tree)
+
+    def test_get_tree_with_show_sizes(self):
+        """Test get_tree API with show_sizes parameter."""
+        from loppers import get_tree
+
+        with tempfile.TemporaryDirectory() as tmpdir:
+            root = Path(tmpdir)
+            file1 = root / "file1.py"
+            file1.write_text("x" * 100)  # 100 bytes
+            file2 = root / "file2.txt"
+            file2.write_text("y" * 2048)  # 2048 bytes
+
+            # With sizes
+            tree_with_sizes = get_tree(root, show_sizes=True)
+            self.assertIn("file1.py", tree_with_sizes)
+            self.assertIn("file2.txt", tree_with_sizes)
+            self.assertIn("(100B)", tree_with_sizes)
+            self.assertIn("(2.0KB)", tree_with_sizes)
+
+            # Without sizes (default)
+            tree_no_sizes = get_tree(root, show_sizes=False)
+            self.assertIn("file1.py", tree_no_sizes)
+            self.assertNotIn("(100B)", tree_no_sizes)
+
+    def test_get_tree_with_sizes_and_collapse(self):
+        """Test get_tree with both show_sizes and collapse_single_dirs."""
+        from loppers import get_tree
+
+        with tempfile.TemporaryDirectory() as tmpdir:
+            root = Path(tmpdir)
+            deep_dir = root / "a" / "b" / "c"
+            deep_dir.mkdir(parents=True)
+            file = deep_dir / "file.py"
+            file.write_text("x" * 1024)
+
+            tree = get_tree(
+                root,
+                show_sizes=True,
+                collapse_single_dirs=True,
+            )
+            # Should collapse the path
+            self.assertIn("a/b/c", tree)
+            # Should show size
+            self.assertIn("(1.0KB)", tree)
 
 
 if __name__ == "__main__":
